@@ -31,16 +31,12 @@
    with compression method 2&3  :		56,413
    with method 1&2&3 and fixed delta row bug:	53,814
    with blank rows sent as such :		48,379
-   
 */
 
-/* Do not include printer specific stuff in this driver. This driver does
-   not cares about the printer model, the only thing that matters is
-   the printer caracteristics. Chema */
-
 #define DEBUG
-#define _PCL_DPI_TEMP 300
+/*
 #define DITHER_HACK
+*/
 #define RED_VALUE 31
 #define GREEN_VALUE 61
 #define BLUE_VALUE 8
@@ -60,113 +56,15 @@
 #include <config.h>
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
+#include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print-pclr.h>
-#include <libgnomeprint/gnome-print-compress.h>
-
-/* - - - Needed for timing operations, remove later - - -*/
-#include <time.h>
-#define MAX_TIME_STRING 100
-typedef struct {
-	clock_t begin_clock, save_clock;
-	time_t  begin_time, save_time;
-} time_keeper;
-static time_keeper tk;
-void   start_time (void);
-double print_time (void);
-/* - - - Needed for timing operations, remove later - - -*/
-
-typedef enum {
-	PCL_COLOR_MODE_BLACK,
-	PCL_COLOR_MODE_CMY,
-	PCL_COLOR_MODE_CMYK,
-	PCL_COLOR_MODE_RGB,
-} PCLColorModes;
-
-typedef enum {
-	PCL_COMPRESSION_METHOD_NO_COMPRESSION, /* 0 */
-	PCL_COMPRESSION_METHOD_RLC,            /* 1 */
-	PCL_COMPRESSION_METHOD_TIFF,           /* 2 */
-	PCL_COMPRESSION_METHOD_DROW,           /* 3 Delta row */
-	PLC_COMPRESSION_METHOD_RESERVED_4,     /* 4 */
-	PLC_COMPRESSION_METHOD_RESERVED_5,     /* 5 */
-	PLC_COMPRESSION_METHOD_RESERVED_6,     /* 6 */
-	PLC_COMPRESSION_METHOD_RESERVED_7,     /* 7 */
-	PLC_COMPRESSION_METHOD_RESERVED_8,     /* 8 */
-	PLC_COMPRESSION_METHOD_RESERVED_9,     /* 9 */
-} PCLCompressionModes;
-
-typedef struct _PCLJobData {
-	gint uel;
-
-	float margin_top;
-	float margin_bottom;
-	float margin_left;
-	float margin_right;
-
-	gint paper_run_dpi;
-	gint paper_width_dpi;
-	gint x_dpi;
-	gint y_dpi;
-
-        gint color_mode;
-	gint simplex_duplex;
-	gint paper_size_capable;
-	gint paper_size;
-        gint media_type;
-	gint orientation;
-
-	gint number_of_planes;
-	
-	gint new_end_graphics_method;
-
-	gint  calibrate;
-	float calibration_brightness;
-	float calibration_contrast;
-	float calibration_red;
-	float calibration_green;
-	float calibration_blue;
-	float calibration_gamma;
-	float calibration_density;
-	float calibration_saturation;
-} PCLJobData;
+#include <libgnomeprint/gnome-print-pclc.h>
+#include <libgnomeprint/gnome-print-encode.h>
 
 PCLJobData * jobdata;
 static GnomePrintRGBPClass *pclr_parent_class;
 
-       void pclr_dump_structure_values_to_console (void);
-       void pclr_new_job_data (void);
-
-
-/*----------  timing functs, remove later ----------- */ 
-void start_time (void)
-{
-	tk.begin_clock = tk.save_clock = clock ();
-	tk.begin_time = tk.save_time = time (NULL);
-}
-
-double
-print_time (void)
-{
-	gchar s1 [MAX_TIME_STRING], s2 [MAX_TIME_STRING];
-	gint  field_width, n1, n2;
-	gdouble clocks_per_second = (double) CLOCKS_PER_SEC, user_time, real_time;
-
-	user_time = (clock() - tk.save_clock) / clocks_per_second;
-	real_time = difftime (time (NULL), tk.save_time);
-	tk.save_clock = clock ();
-	tk.save_time = time (NULL);
-
-	/* print the values ... */
-	n1 = sprintf (s1, "%.5f", user_time);
-	n2 = sprintf (s2, "%.5f", real_time);
-	field_width = (n1 > n2)?n1:n2;
-	g_print ("%s%*.5f%s\n%s%*.5f%s\n\n",
-		 "User time : ", field_width, user_time, " seconds",
-		 "Real time : ", field_width, real_time, " seconds");
-	return user_time;
-}
-/*----------  End timing functions  ----------- */
-
 static int
 pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 {
@@ -180,7 +78,7 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 	gint rowbuff_rlc_size    [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
 	gint rowbuff_tiff_size   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
 	gint rowbuff_drow_size   [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
-	gint rowbuff_drow_2_size; /* We calculate the seed with plane #1.
+	gint rowbuff_drow_2_size = 0; /* We calculate the seed with plane #1.
 				   and modify with the seed source Esc code */ 
 	guchar *rowbuff        [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
 	guchar *rowbuff_rlc    [_GNOME_PRINT_PCL_MAX_PLANES_SUPPORTED];
@@ -243,18 +141,31 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 	/* Let's ask for some memory. TODO : check for success */
 	for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 	{
-		rowbuff_size   [current_plane] = (int) ((cols-1)/8) + 1;
-		rowseed        [current_plane] = g_malloc ( rowbuff_size [current_plane] + 1);
-		/* the row of pixels */
-		rowbuff        [current_plane] = g_malloc ( rowbuff_size [current_plane] + 1);
+		gint size_row, size_needed;
 
-		/* the same row but with compression method 1, Run Length Coding. WCS : size * 2 */ 
-		rowbuff_rlc    [current_plane] = g_malloc ( rowbuff_size [current_plane] * 2 + 1);
-		/* the same row but with compression method 2. FIXME : WCS ?  */ 
-		rowbuff_tiff   [current_plane] = g_malloc ( rowbuff_size [current_plane] * 2);
-		/* that same row with compression 3, Delta Row. WCS: size + int(size/8) + 1*/
-		rowbuff_drow   [current_plane] = g_malloc ( rowbuff_size [current_plane]  +
-						    (gint)(rowbuff_size [current_plane] /8) + 2);
+		rowbuff_size   [current_plane] = (int) ((cols-1)/8) + 1;
+
+		size_row = rowbuff_size [current_plane];
+
+		/* the seed row needed for delta row */
+		size_needed = size_row + 1;
+		rowseed       [current_plane] = g_malloc (size_needed);
+		/* the row of pixels */
+		size_needed = size_row + 1;
+		rowbuff       [current_plane] = g_malloc (size_needed);
+		memset (rowbuff [current_plane], 0, size_needed);
+		/* the same row but with compression method 1, Run Length Coding.  */
+		size_needed = gnome_print_encode_rlc_wcs (size_row);
+		rowbuff_rlc    [current_plane] = g_malloc (size_needed);
+		memset (rowbuff_rlc [current_plane], 0, size_needed);
+		/* the same row but with compression method 2. */ 
+		size_needed = gnome_print_encode_tiff_wcs (size_row);
+		rowbuff_tiff   [current_plane] = g_malloc (size_needed);
+		memset (rowbuff_tiff [current_plane], 0, size_needed);
+		/* that same row with compression 3, Delta Row*/
+		size_needed = gnome_print_encode_drow_wcs (size_row);
+		rowbuff_drow   [current_plane] = g_malloc (size_needed);
+		memset (rowbuff_drow [current_plane], 0, size_needed);
 		last_compression_method_used = PCL_COMPRESSION_METHOD_NO_COMPRESSION;
 		blank_rows_count = 0;
 	}
@@ -334,18 +245,31 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 			k++;
 		}
 
+		if (j%24 != 0)
+		{
+			for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
+			{
+				guchar * temp;
+				temp = (guchar *) rowbuff [current_plane];
+				temp [offset_in_row] = byte [current_plane];
+				byte [current_plane] = 0;
+			}
+			offset_in_row++;
+			k = 0;
+		}
+
 		current_plane=0;
 
 		/* We cant send blank lines in RBG mode since 000 in RBG = black.
 		   if we ever implement CMY or CMYK we should enable this optimization.
-		   for now, only use it when Color = K */ 
+		   for now, only use it when Color = K */
 		if (jobdata->number_of_planes==1)
 		{
 			/* Process blank lines as such */
 			blank_line = TRUE; /* All lines are inocent until proven guilty */ 
 			for (current_plane=0; current_plane < jobdata->number_of_planes; current_plane++)
 			{
-				if (!gnome_print_compress_blank (rowbuff [current_plane], rowbuff_size [current_plane]))
+				if (!gnome_print_encode_blank (rowbuff [current_plane], rowbuff_size [current_plane]))
 				{
 					blank_line=FALSE;
 				}
@@ -382,7 +306,7 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 			if (i!=0)
 			{
 				rowbuff_drow_size [current_plane] =
-					gnome_print_compress_drow ( (guchar *) rowbuff [current_plane],
+					gnome_print_encode_drow ( (guchar *) rowbuff [current_plane],
 								    (guchar *) rowbuff_drow [current_plane],
 								    rowbuff_size [current_plane],
 								    (guchar *) rowseed [current_plane]);
@@ -392,7 +316,7 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 				if (current_plane > 0)
 				{
 					rowbuff_drow_2_size  =
-						gnome_print_compress_drow ( (guchar *) rowbuff [current_plane],
+						gnome_print_encode_drow ( (guchar *) rowbuff [current_plane],
 									    (guchar *) rowbuff_drow_2 ,
 									    rowbuff_size [current_plane],
 									    (guchar *) rowseed [current_plane-1]);
@@ -418,11 +342,11 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 			if (rowbuff_drow_size [current_plane]!=0)
 			{
 				rowbuff_rlc_size [current_plane] =
-					gnome_print_compress_rlc  ( (guchar *) rowbuff [current_plane],
+					gnome_print_encode_rlc  ( (guchar *) rowbuff [current_plane],
 								    (guchar *) rowbuff_rlc [current_plane],
 								    rowbuff_size [current_plane] );
 				rowbuff_tiff_size [current_plane] =
-					gnome_print_compress_tiff ( (guchar *) rowbuff [current_plane],
+					gnome_print_encode_tiff ( (guchar *) rowbuff [current_plane],
 								    (guchar *) rowbuff_tiff [current_plane],
 								    rowbuff_size [current_plane] );
 			}
@@ -554,6 +478,7 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 		} /* For current_plane */ 
 	}/*for (i=0;i<rows;i++)*/
 
+	/* Free the memory used */
 	for (current_plane=0;current_plane < jobdata->number_of_planes; current_plane++)
 	{
 		g_free (rowseed      [current_plane]);
@@ -562,6 +487,7 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 		g_free (rowbuff_tiff [current_plane]);
 		g_free (rowbuff_drow [current_plane]);
 	}
+	g_free (rowbuff_drow_2);
 
 	/* End Raster Graphics */
 	status =+ gnome_print_context_fprintf ( pc, ESC_ "*rC", cols);
@@ -570,192 +496,20 @@ pclr_print_band (GnomePrintRGBP *rgbp, guchar *rgb_buffer, ArtIRect *rect)
 }
 
 
-/* This is the structure that will be provided TO the driver
-   by gnome-print, and will be filled in this order :
-
-1.  gnome-print will call PCL_create_jobdata()
-    and the default values will be loaded.
-2.  gnome-print will change the default values with values
-    specific to that printer.
-3.  gnome-print will the modify the values that the user
-    has configured for that printer.
-4.  The application and the print dialog will load the
-    job specific parameters */
-
-
-void
-pclr_new_job_data (void)
-{
-/* TODO : Divide this data in what is specific to this driver
-   and what is common across all drivers. Chema */
-	jobdata = g_new0 (PCLJobData, 1);
-
-	jobdata->uel = TRUE;
-/* TRUE = the printer understands the Universal Exit Languaje command
-   most LJ printers do except for the III IIID & IIIP. */
-
-	jobdata->margin_top = 1/6;
-	jobdata->margin_bottom = 1/6;
-	jobdata->margin_left = 1/6;
-	jobdata->margin_right = 1/6;
-/* Printable Area Margins in inches. ([1] Page 2.7.) This seems way too optimistic
-   for Lasers. For Deskjet margins are greater. Need to test in the field. */
-
-	jobdata->paper_run_dpi = _PCL_DPI_TEMP;
-	jobdata->paper_width_dpi = _PCL_DPI_TEMP;
-	jobdata->x_dpi=jobdata->paper_width_dpi;
-	jobdata->y_dpi=jobdata->paper_run_dpi;
-/* Do not use xdpi and ydpi, run and width are clear because it's independent of
-   orientation, we should load xdip and ydpi with corresponding values */
-
-	jobdata->simplex_duplex = -1;
-/* -1   = printer does not supports it,
-    0   = printer supports it but no duplex has been chosen
-    1,2 = duplex printing ([1]Page 4-5) long and short edge
-          long edge is NOT supported by all duplex capable printers*/
-
-	jobdata->paper_size_capable=FALSE;
-/* Althou most printers do, the default is that this printer does
-   not understand the paper_size Escape code. Most profiles should
-   have this enabled, since most printers can select paper size*/
-
-	jobdata->paper_size = 2;
-/* Letter is default, maybe we should fall bacj to gnome_print_default_paper.
-   For printers that support custom paper sizes we can choose any size */
-
-        jobdata->media_type = -1;
-/* -1 = Printer does not understand the mediatype command.
-   other = mediatype.
-   LaserJets do not support media type [3]Page34 */
-
-	jobdata->orientation = 0;
-/* 0 = Portrait
-   1 = Landscape
-   2 = Reverse Portrait
-   3 = Reverse Landscape */
-
-#ifdef ENABLE_COLORS
-        jobdata->color_mode = PCL_COLOR_MODE_RGB;
-	jobdata->number_of_planes = 3;
-#else	
-        jobdata->color_mode = PCL_COLOR_MODE_BLACK;
-	jobdata->number_of_planes = 1;
-#endif	
-/* See enum above */
-/* Number of planes or colors. 1 for black
-   and 3 for RGB */
-	
-	jobdata->calibrate = TRUE;
-/* The default for calibration if off, but we
-   still need to load the calibration parameters.
-   The calibration parameters will range from 1 to 200
-   with 100 beeing the default, we might want to change
-   this latter*/
-	jobdata->calibration_brightness=100;
-	jobdata->calibration_contrast=100;
-	jobdata->calibration_red=100;
-	jobdata->calibration_green=100;
-	jobdata->calibration_blue=100;
-	jobdata->calibration_gamma=100;
-	jobdata->calibration_density=100;
-	jobdata->calibration_saturation=100;
-
-/* The end raster graphics command  was changed for some stupid
-   and unkown reason, I am not aware of any printers that NEED
-   to use the new method. Set this to True if needed. Chema */
-	jobdata->new_end_graphics_method=FALSE;
-}                                                  
-
-
-void
-pclr_dump_structure_values_to_console (void)
-{
-	/* We print all the parameters as a debuging tool ....*/
-	g_print("Here are the parameters for this print job :\n");
-
-	if (jobdata->uel)
-		g_print("This printer DOES understand UEL\n");
-	else
-		g_print("This printer DOES NOT understands UEL\n");
-
-	switch (jobdata->simplex_duplex){
-	case -1:
-		g_print("This printer does not support Duplexing\n");
-		break;
-	case 0:
-		g_print("This printer supports duplexing but it is not needed\n");
-		break;
-	case 1:
-	case 2:
-		g_print("This job will be printed with duplexing\n");
-		break;
-	}
-
-	if ((jobdata->orientation%2)==0)
-		g_print("Portrait or reverse portrait orientation\n");
-	else
-		g_print("Landscape or reverse Landscape  orientation\n");
-
-	if (jobdata->paper_size_capable)
-		g_print("You can't select paper size for this printer\n");
-	else
-		g_print("The paper size selcted (in HPL numbers) is : %i\n", jobdata->paper_size);
-	
-	if (jobdata->media_type == -1)
-		g_print("You can't select media type for this printer\n");
-	else
-		g_print("The media type selected is number : %i\n", jobdata->media_type);
-
-	switch (jobdata->color_mode){
-	case PCL_COLOR_MODE_BLACK:
-		g_print("Color mode : black\n");
-		break;
-	case PCL_COLOR_MODE_CMY:
-		g_print("Color mode : CMY\n");
-		break;
-	case PCL_COLOR_MODE_CMYK:
-		g_print("Color mode : CMYK\n");
-		break;
-	default:
-		g_print("Color mode undefined\n");
-		break;
-	}
-
-	g_print("Reslution is : %d in paper direction and %d in paper width\n",
-		jobdata->paper_run_dpi, jobdata->paper_width_dpi);
-
-	if (jobdata->calibrate == TRUE)
-	{
-		g_print("The calibration parameters have been set as follows :\n");
-		g_print("Brightness : %f\n", jobdata->calibration_brightness);
-		g_print("Contrast   : %f\n", jobdata->calibration_contrast);
-		g_print("Red        : %f\n", jobdata->calibration_red);
-		g_print("Green      : %f\n", jobdata->calibration_green);
-		g_print("Blue       : %f\n", jobdata->calibration_blue);
-		g_print("Gamma      : %f\n", jobdata->calibration_gamma);
-		g_print("Density    : %f\n", jobdata->calibration_density);
-		g_print("Saturation : %f\n", jobdata->calibration_saturation);
-	}
-	else
-		g_print("No calibration parameters can be set for this printer\n");
-	
-	g_print("\n\n");
-}
-
 static int
 pclr_job_start (GnomePrintContext *pc)
 {
 	int status;
 
 	g_print("\n\nJob start ....... STARTING TIMER ..\n");
-	start_time();
+	gnome_print_encode_timer_start ();
 
 	/* Here we load the job data, but this will be provided for the
 	   driver. Chema*/
-	pclr_new_job_data ();
+	pclc_new_job_data ();
 
 #ifdef DEBUG
-	pclr_dump_structure_values_to_console();
+	pclc_dump_structure_values_to_console();
 #endif DEBUG
 
 	/* Here we start the print job */
@@ -833,14 +587,45 @@ pclr_job_start (GnomePrintContext *pc)
 	return 0;
 }
 
+static int
+pclr_page_end (GnomePrintRGBP *rgbp)
+{
+	gint status;
+	GnomePrintContext *pc;
+
+	g_return_val_if_fail (GNOME_IS_PRINT_RGBP (rgbp), -1);
+	pc = GNOME_PRINT_CONTEXT (rgbp);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	
+	status =+ gnome_print_context_fprintf ( pc, ESC_ "&l0H", 0); 
+	
+	return 0;
+}
+
+#ifdef KILL_COMPILE_WARNING
+static int
+pclr_page_begin (GnomePrintContext *pc)
+{
+	g_print ("Page begin\n");
+	return 0;
+}
+#endif
+
+static int
+pclr_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
+{
+	/* Nothing.  */
+	return 0;
+}
+
 static int
 pclr_close (GnomePrintContext *pc)
 {
-	/* FIXME : We need to job separation command also */
+	/* FIXME : We need to send job separation command also */
 	gnome_print_context_fprintf ( pc, ESC_ "E");
 
 	/* REMOVE LATER */
-	print_time();
+	gnome_print_encode_timer_end ();
 	
 	return gnome_print_context_close_file (pc);
 }
@@ -854,8 +639,11 @@ pclr_class_init (GtkObjectClass *object_class)
 	pclr_parent_class = gtk_type_class (gnome_print_rgbp_get_type ());
 
 	rgbp_class->print_band = pclr_print_band;
+	rgbp_class->page_end   = pclr_page_end;
 
 	pc_class->close = pclr_close;
+	pc_class->beginpage = pclr_beginpage;
+
 }
 
 GnomePrintPCLR *
@@ -895,7 +683,7 @@ gnome_print_pclr_new (GnomePrinter *printer, const char *paper_size, int dpi)
 	if (!gnome_print_pclr_construct (pclr, printer, paper_info, dpi))
 		gtk_object_unref (GTK_OBJECT (pclr));
 
-	/* Is this ok ? chema. */
+	/* Is this the correct location for this call ? Chema. */
 	pclr_job_start (GNOME_PRINT_CONTEXT (pclr));
 	
 	return GNOME_PRINT_CONTEXT (pclr);
