@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * gnome-print.c: Main GnomePrint API
  *
@@ -6,16 +7,40 @@
  *   Miguel de Icaza (miguel@kernel.org)
  *   Lauris Kaplinski <lauris@ariman.ee>
  */
+/* Must include these two first */
+#include "config.h"
+#include <libgnomeprint/gnome-print-i18n.h>
+
+#include <string.h>
+#include <locale.h>
+#include <unicode.h>
+
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_vpath.h>
 #include <libart_lgpl/art_bpath.h>
+/* gnome.h is only needed for g_concat_string_and_file */
+#include <gnome.h>
+
+#include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-private.h>
+
+/* Gnome Print Drivres */
 #include <libgnomeprint/gnome-print-ps.h>
+#include <libgnomeprint/gnome-print-ps2.h>
+#include <libgnomeprint/gnome-print-pdf.h>
+#include <libgnomeprint/gnome-print-pclr.h>
+#include <libgnomeprint/gnome-print-pclv.h>
 #include <libgnomeprint/gnome-print-pixbuf.h>
-#include <string.h>
-#include <locale.h>
+#include <libgnomeprint/gnome-print-frgba.h>
+
+#include <libgnomeprint/gnome-font-private.h>
+#include <libgnomeprint/gnome-pgl.h>
+#include <libgnomeprint/gnome-pgl-private.h>
+#include <libgnomeprint/gp-ps-unicode.h>
+
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 
@@ -60,7 +85,8 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	parent_class = gtk_type_class (gtk_object_get_type ());
 
 	object_class->finalize = gnome_print_context_finalize;
-
+#if 0
+	/* Well - according to spec GtkObjectClass if zeroed */
 	class->newpath = NULL;
 	class->moveto = NULL;
 	class->lineto = NULL;
@@ -77,9 +103,11 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	class->strokepath = NULL;
 	class->stroke = NULL;
 	class->setfont = NULL;
-	class->show = NULL;
+	class->show_ucs4 = NULL;
 	class->concat = NULL;
+#ifdef SETMATRIX_DEPRECATED
 	class->setmatrix = NULL;
+#endif
 	class->gsave = NULL;
 	class->grestore = NULL;
 	class->clip = NULL;
@@ -92,6 +120,7 @@ gnome_print_context_class_init (GnomePrintContextClass *class)
 	class->close = NULL;
 	class->setopacity = NULL;
 	class->rgbaimage = NULL;
+#endif
 }
 
 static void
@@ -107,13 +136,38 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 	g_return_val_if_fail (GNOME_IS_PRINTER (printer), NULL);
 	g_return_val_if_fail (paper_size != NULL, NULL);
 
+	/* This is just TEMPORARY !*/
+
 	if (strcmp (printer->driver, "gnome-print-ps") == 0) {
+		GnomePrintPs *ps = gnome_print_ps_new (printer);
+		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps);
+		return frgba ? frgba : NULL;
+	}
+	if (strcmp (printer->driver, "gnome-print-ps-rgb") == 0) {
 		GnomePrintPs *ps = gnome_print_ps_new (printer);
 		return ps ? GNOME_PRINT_CONTEXT (ps) : NULL;
 	}
+	if (strcmp (printer->driver, "gnome-print-ps2") == 0) {
+		GnomePrintPs2 *ps2 = gnome_print_ps2_new (printer);
+		GnomePrintContext *frgba = gnome_print_frgba_new ((GnomePrintContext *) ps2);
+		return frgba ? frgba : NULL;
+	}
+	if (strcmp (printer->driver, "gnome-print-pdf") == 0) {
+		GnomePrintPdf *pdf = gnome_print_pdf_new (printer);
+		return pdf ? GNOME_PRINT_CONTEXT (pdf) : NULL;
+	}
 
+	if (strcmp (printer->driver, "gnome-print-pclr") == 0) {
+		return gnome_print_pclr_new (printer, paper_size, 300);
+	}
+	if (strcmp (printer->driver, "gnome-print-pclv") == 0) {
+		return gnome_print_pclv_new (printer, paper_size, 300);
+	}
+
+#if 0
 	if (strncmp (printer->driver, "gnome-print-uni", strlen ("gnome-print-uni")) == 0)
 		return gnome_print_pixbuf_new (printer, paper_size, 75);
+#endif
 
 	return NULL;
 }
@@ -302,11 +356,69 @@ gnome_print_setfont (GnomePrintContext *pc, GnomeFont *font)
 int
 gnome_print_show (GnomePrintContext *pc, char const *text)
 {
+	static gboolean uerror = FALSE;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (text != NULL, -1);
+	g_return_val_if_fail (!uerror, -1);
 
-	return PRINT_CLASS(pc)->show (pc, text);
+	if (strlen (text) < 1) return 0;
+
+	if (PRINT_CLASS (pc)->show_ucs4) {
+		static gboolean uinit = FALSE;
+		static unicode_iconv_t uiconv = (unicode_iconv_t) -1;
+		const gchar * ibuf;
+		guint32 * buf;
+		gchar * obuf;
+		size_t ilen, olen;
+		ssize_t converted;
+		gint ret;
+
+		if (!uinit) {
+			unicode_init ();
+			uiconv = unicode_iconv_open ("UCS4", "UTF-8");
+			if (uiconv == (unicode_iconv_t) -1) {
+				g_warning ("Could not initilize unicode convertion");
+				uerror = TRUE;
+				return -1;
+			}
+			uinit = TRUE;
+		}
+
+		ret = -1;
+
+		ilen = strlen (text);
+		ibuf = (const gchar *) text;
+		olen = ilen * 2 * sizeof (guint32);
+		buf = g_new (guint32, 2 * ilen);
+		obuf = (gchar *) buf;
+		converted = unicode_iconv (uiconv, &ibuf, &ilen, &obuf, &olen);
+
+		if (converted >= 0) {
+			gint length;
+			length = (guint32 *) obuf - buf;
+			ret = (* PRINT_CLASS (pc)->show_ucs4) (pc, buf, length);
+		}
+		g_free (buf);
+		return ret;
+	}
+
+	return -1;
+}
+
+int
+gnome_print_show_ucs4 (GnomePrintContext * pc, guint32 * buf, gint length)
+{
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (buf != NULL, -1);
+	g_return_val_if_fail (length > 0, -1);
+
+	if (PRINT_CLASS (pc)->show_ucs4)
+		return (* PRINT_CLASS (pc)->show_ucs4) (pc, buf, length);
+
+	return -1;
 }
 
 int
@@ -319,6 +431,36 @@ gnome_print_concat (GnomePrintContext *pc, const double matrix[6])
 	return PRINT_CLASS(pc)->concat (pc, matrix);
 }
 
+#if 0
+/**
+ * gnome_print_concat_optimize:
+ * @gpc: 
+ * @: 
+ * 
+ * This method should be use when apps need to concat a matrix
+ * that will be used on all of the page. For example a scaling factor or
+ * landscape rotation+translation. No harm is done if this method is a) not
+ * used or b) not used correctly. This function provides a mayor optimization
+ * when scaling is to be done in the whole page and when the page needs to be
+ * landscape rotated.
+ **/
+int
+gnome_print_concat_optimize (GnomePrintContext *pc, const double matrix[6])
+{
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (matrix != NULL, -1);
+
+	g_warning ("DO NOT use this function. It is experimental and will be removed\n");
+	
+	if (PRINT_CLASS (pc)->concat_optimize != NULL)
+		return PRINT_CLASS(pc)->concat_optimize (pc, matrix);
+	else
+		return PRINT_CLASS(pc)->concat (pc, matrix);
+}
+#endif
+
+#ifdef SETMATRIX_DEPRECATED
 int
 gnome_print_setmatrix (GnomePrintContext *pc, const double matrix[6])
 {
@@ -328,6 +470,7 @@ gnome_print_setmatrix (GnomePrintContext *pc, const double matrix[6])
 
 	return PRINT_CLASS(pc)->setmatrix (pc, matrix);
 }
+#endif
 
 int
 gnome_print_gsave (GnomePrintContext *pc)
@@ -430,6 +573,24 @@ gnome_print_rgbaimage (GnomePrintContext *pc, const char *data, int width, int h
 	return ret;
 }
 
+int
+gnome_print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
+{
+       if (gdk_pixbuf_get_has_alpha (pixbuf))
+               gnome_print_rgbaimage  (pc,
+                                       gdk_pixbuf_get_pixels    (pixbuf),
+                                       gdk_pixbuf_get_width     (pixbuf),
+                                       gdk_pixbuf_get_height    (pixbuf),
+                                       gdk_pixbuf_get_rowstride (pixbuf));
+       else
+               gnome_print_rgbimage  (pc,
+                                      gdk_pixbuf_get_pixels    (pixbuf),
+                                      gdk_pixbuf_get_width     (pixbuf),
+                                      gdk_pixbuf_get_height    (pixbuf),
+                                      gdk_pixbuf_get_rowstride (pixbuf));
+	return 1;
+}
+
 int
 gnome_print_textline (GnomePrintContext *pc, GnomeTextLine *line)
 {
@@ -537,6 +698,12 @@ gnome_print_context_open_file (GnomePrintContext *pc, const char *filename)
 	}
 	else
 	{
+		/* This is not very intelligent, but it worked for me */
+
+		if ((filename[0] == '~') && (filename[1] == '/'))
+		{
+			filename = g_concat_dir_and_file (g_get_home_dir (), &filename[2]);
+		}
 		pc->f = fopen (filename, "w");
 		pc->is_pipe = FALSE;
 	}
@@ -662,6 +829,8 @@ gnome_print_bpath (GnomePrintContext * gpc, ArtBpath * bpath, gboolean append)
 
 }
 
+
+
 void
 gnome_print_vpath (GnomePrintContext * gpc, ArtVpath * vpath, gboolean append)
 {
@@ -716,3 +885,66 @@ gnome_print_vpath (GnomePrintContext * gpc, ArtVpath * vpath, gboolean append)
 		gnome_print_closepath (gpc);
 
 }
+
+/*
+ * WARNING: EXPERIMENTAL
+ */
+
+int
+gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
+{
+	g_return_val_if_fail (pc != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (glyphlist != NULL, -1);
+	g_return_val_if_fail (GNOME_IS_GLYPHLIST (glyphlist), -1);
+
+	if (((GnomePrintContextClass *) ((GtkObject *) pc)->klass)->glyphlist) {
+		return (* ((GnomePrintContextClass *) ((GtkObject *) pc)->klass)->glyphlist) (pc, glyphlist);
+	} else {
+		GnomePosGlyphList * pgl;
+		gdouble affine[6];
+		GSList * l;
+		art_affine_identity (affine);
+		pgl = gnome_pgl_from_gl (glyphlist, affine, 0);
+		for (l = pgl->strings; l != NULL; l = l->next) {
+			GnomePosString *string;
+			GnomeFont *font;
+			GnomeFontFace *face;
+			gint i;
+			string = (GnomePosString *) l->data;
+			font = (GnomeFont *) gnome_rfont_get_font (string->rfont);
+			face = (GnomeFontFace *) gnome_font_get_face (font);
+			gnome_print_setfont (pc, font);
+			for (i = 0; i < string->length; i++) {
+				gdouble r, g, b, a;
+				const gchar *psname;
+				gint ucschar;
+				/* fixme: We need currentpoint here :( */
+				gnome_print_moveto (pc, string->glyphs[i].x, string->glyphs[i].y);
+				r = ((string->glyphs[i].color >> 24) & 0xff) / 255.0;
+				g = ((string->glyphs[i].color >> 16) & 0xff) / 255.0;
+				b = ((string->glyphs[i].color >>  8) & 0xff) / 255.0;
+				a = ((string->glyphs[i].color >>  0) & 0xff) / 255.0;
+				gnome_print_setrgbcolor (pc, r, g, b);
+				gnome_print_setopacity (pc, a);
+				psname = gnome_font_face_get_glyph_ps_name (face, string->glyphs[i].glyph);
+				ucschar = GINT_TO_BE (gp_unicode_from_ps (psname));
+				gnome_print_show_ucs4 (pc, &ucschar, 1);
+			}
+		}
+		gnome_pgl_destroy (pgl);
+	}
+
+	return 0;
+}
+
+
+
+
+
+
+
+
+
+
+
