@@ -5,6 +5,8 @@
 
 #include <glib.h>
 
+#include <libgnomeprint/gp-ps-unicode.h>
+
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_rect.h>
 #include <libart_lgpl/art_vpath.h>
@@ -2816,7 +2818,11 @@ struct _Gt1LoadedFont {
   Font_Info *fi;
   MunchedFontInfo *mfi;
   Gt1Dict *fontdict;
+#if 0
   Gt1NameId *encoding;
+#else
+	GHashTable *encoding;
+#endif
   Gt1NameId id_charstrings;
   GHashTable *glyph_outlines;
 };
@@ -2992,9 +2998,12 @@ gt1_load_font (const char *filename)
   int bytes_read;
   Gt1TokenContext *tc;
   Gt1PSContext *psc;
+  Gt1NameContext *nc;
 
   char *flat;
 
+  GHashTable *encoding;
+#if 0
   Gt1NameId *encode_table_1,
             *encode_table_2,
             *encode_table_3,
@@ -3002,58 +3011,9 @@ gt1_load_font (const char *filename)
             *enc_std,
             *enc_isolatin,
             *enc_utf8;
+#endif
   int i;
 
-  /* Encode table 1 from 040 to 0176, ascii */ 
-  static char *  encode_table_1_names[] = {
-/*4x */"space",      "exclam",     "quotedbl",   "numbersign", "dollar",     "percent",     "ampersand",  "quoteright",
-/*5x */"parenleft",  "parenright", "asterisk",   "plus",       "comma",      "hyphen",      "period",     "slash",
-/*6x */"zero",       "one",        "two",        "three",      "four",       "five",        "six",        "seven",
-/*7x */"eight",      "nine",       "colon",      "semicolon",  "less",       "equal",       "greater",    "question",
-/*10x*/"at",         "A",          "B",          "C",          "D",          "E",           "F",          "G",
-/*11x*/"H",          "I",          "J",          "K",          "L",          "M",           "N",          "O",
-/*12x*/"P",          "Q",          "R",          "S",          "T",          "U",           "V",          "W",
-/*13x*/"X",          "Y",          "Z",          "bracketleft","backslash",  "bracketright","asciicircum","underscore",
-/*14x*/"quoteleft",  "a",          "b",          "c",          "d",          "e",           "f",          "g",
-/*15x*/"h",          "i",          "j",          "k",          "l",          "m",           "n",          "o",
-/*16x*/"p",          "q",          "r",          "s",          "t",          "u",           "v",          "w",
-/*170*/"x",          "y",          "z",          "braceleft",  "bar",        "braceright",  "asciitilde"};
-   
-  /* From 0241 ( 161) to 0320 ( 208 ) */ 
-static char *  encode_table_2_names[] = {
-/*24x*/                 "exclamdown",   "cent",          "sterling",      "fraction",      "yen",             "florin",    "section",
-/*25x*/"currency",      "quotesingle",  "quotedblleft",  "guillemotleft", "guilsinglleft", "guilsinglright",  "fi",        "fl",
-/*26x*/".notdef",       "endash",       "dagger",        "daggerdbl",     "periodcentered",".notdef",         "paragraph", "bullet",
-/*27x*/"quotesinglbase","quotedblbase", "quotedblright", "guillemotright","ellipsis",      "perthousand",     ".notdef",   "questiondown",
-/*30x*/".notdef",       "grave",        "acute",         "circumflex",    "tilde",         "macron",          "breve",     "dotaccent",
-/*31x*/"dieresis",      ".notdef",      "ring",          "cedilla",       ".notdef",       "hungarumlaut",    "ogonek",    "caron",
-/*32x*/"emdash",        "123456789012345678"};
-
-/* Table # 3 : from 0220 ( 144 ) to 0377 ( 255 ) */
-static char *  encode_table_3_names[] = {
-/*22x*/"dotlessi",      "grave",        "acute",         "circumflex",    "tilde",         "macron",          "breve",        "dotaccent",
-/*23x*/"dieresis",      ".notdef",      "ring",          "cedilla",       ".notdef",       "hungarumlaut",    "ogonek",       "caron",
-/*24x*/"space",         "exclamdown",   "cent",          "sterling",      "currency",      "yen",             "brokenbar",    "section",
-/*25x*/"dieresis",      "copyright",    "ordfeminine",   "guillemotleft", "logicalnot",    "hyphen",          "registered",   "macron",
-/*26x*/"degree",        "plusminus",    "twosuperior",   "threesuperior", "acute",         "mu",              "paragraph",    "periodcentered",
-/*27x*/"cedilla",       "onesuperior",  "ordmasculine",  "guillemotright","onequarter",    "onehalf",         "threequarters","questiondown",
-/*30x*/"Agrave",        "Aacute",       "Acircumflex",   "Atilde",        "Adieresis",     "Aring",           "AE",           "Ccedilla",
-/*31x*/"Egrave",        "Eacute",       "Ecircumflex",   "Edieresis",     "Igrave",        "Iacute",          "Icircumflex",  "Idieresis",
-/*32x*/"Eth",           "Ntilde",       "Ograve",        "Oacute",        "Ocircumflex",   "Otilde",          "Odieresis",    "multiply",
-/*33x*/"Oslash",        "Ugrave",       "Uacute",        "Ucircumflex",   "Udieresis",     "Yacute",          "Thorn",        "germandbls",
-/*340*/"agrave",        "aacute",       "acircumflex",   "atilde",        "adieresis",     "aring",           "ae",           "ccedilla",
-/*35x*/"egrave",        "eacute",       "ecircumflex",   "edieresis",     "igrave",        "iacute",          "icircumflex",  "idieresis",
-/*36x*/"eth",           "ntilde",       "ograve",        "oacute",        "ocircumflex",   "otilde",          "odieresis",    "divide",
-/*37x*/"oslash",        "ugrave",       "uacute",        "ucircumflex",   "udieresis",     "yacute",          "thorn",        "ydieresis",
-       "123456789012345678"};
-
-/* Table #4 : from 0341 ( 225 ) to 0373 (251), very scattered tho */
-static char *  encode_table_4_names[] = {
-/*34x*/                 "AE",           ".notdef",      ".notdef",       ".notdef",       ".notdef",       ".notdef",         ".notdef",
-/*35x*/"Lslash",        "Oslash",       "OE",           "ordmasculine",  ".notdef",       ".notdef",       ".notdef",         ".notdef",
-/*36x*/".notdef",       "ae",           ".notdef",      ".notdef",       ".notdef",       "dotlessi",      ".notdef",         ".notdef",
-/*37x*/"lslash",        "oslash",       "oe",           "germandbls",    "123456789012345678"};
-
   f = fopen (filename, "rb");
   if (f == NULL) return NULL;
 
@@ -3134,56 +3094,39 @@ static char *  encode_table_4_names[] = {
     }
   gt1_free (afm_filename);
 
-  enc_std        = gt1_new (Gt1NameId, 256);
-  enc_isolatin   = gt1_new (Gt1NameId, 256);
-  enc_utf8       = gt1_new (Gt1NameId, 256);
-  encode_table_1 = gt1_new (Gt1NameId, 256);
-  encode_table_2 = gt1_new (Gt1NameId, 256);
-  encode_table_3 = gt1_new (Gt1NameId, 256);
-  encode_table_4 = gt1_new (Gt1NameId, 256);
+#if 0
+  enc_isolatin   = gt1_new (Gt1NameId, 1024);
+#endif
+  encoding = g_hash_table_new (NULL, NULL);
 
-  for (i = 0; i < 256; i++)
-  {
-    enc_std[i]      = gt1_name_context_intern (psc->nc, ".notdef");
-    enc_isolatin[i] = gt1_name_context_intern (psc->nc, ".notdef");
-    enc_utf8[i]     = gt1_name_context_intern (psc->nc, ".notdef");
-  };
+  nc = psc->nc;
 
-  for (i = 040; i <= 126; i++)
-       encode_table_1[ i] = gt1_name_context_intern (psc->nc, encode_table_1_names[i-040]);
-  for (i = 0214; i <= 0320; i++) 
-       encode_table_2[ i] = gt1_name_context_intern (psc->nc, encode_table_2_names[i-0214]);
-  for (i = 0220; i <= 0377; i++)
-       encode_table_3[ i] = gt1_name_context_intern (psc->nc, encode_table_3_names[i-0220]);
-  for (i = 0341; i <= 0373; i++)
-       encode_table_4[ i] = gt1_name_context_intern (psc->nc, encode_table_4_names[i-0341]);
+  for (i = 0; i < nc->table_size; i++) {
+	  Gt1NameContextHashEntry *nche;
+	  gint unival;
 
-  for( i =  040; i <= 0176; i++)
-  {
-       enc_std[i]      = encode_table_1[i];
-       enc_isolatin[i] = encode_table_1[i];
-       enc_utf8[i]     = encode_table_1[i];
+	  nche = nc->table + i;
+
+	  if (nche->name) {
+		  unival = gp_unicode_from_ps (nche->name);
+		  if (unival > 0) {
+			  g_hash_table_insert (encoding, GINT_TO_POINTER (unival), GINT_TO_POINTER (nche->Gt1NameId));
+		  }
+	  }
   }
 
-  for( i = 0220; i <= 0377; i++)
-       enc_isolatin[i] = encode_table_3[i];
-  for( i = 0240; i <= 0377; i++)
-       enc_utf8[i]    = encode_table_3[i];
-  for( i = 0241; i <= 0320; i++)
-       enc_std[i]      = encode_table_2[i];
-  for( i = 0341; i <= 0373; i++)
-       enc_std[i]      = encode_table_4[i];
-  
-  loaded_font->encoding = enc_isolatin;
+#if 0
+  for (i = 0; i < 1024; i++) {
+	  const gchar *name;
+	  name = gp_ps_from_unicode (i);
+	  if (!name) name = ".notdef";
+	  enc_isolatin[i] = gt1_name_context_intern (psc->nc, name);
+  }
+#endif
+
+  loaded_font->encoding = encoding;
   loaded_font->id_charstrings = gt1_name_context_intern (psc->nc, "CharStrings");
 
-  gt1_free (enc_std);
-  gt1_free (enc_utf8);
-  gt1_free (encode_table_1);
-  gt1_free (encode_table_2);
-  gt1_free (encode_table_3);
-  gt1_free (encode_table_4);
-  
   return loaded_font;
 }
 
@@ -3197,7 +3140,7 @@ gt1_unload_font (Gt1LoadedFont *font)
   glyph_outlines_destroy (font);
   g_hash_table_destroy (font->glyph_outlines);
 
-  gt1_free (font->encoding);
+  g_hash_table_destroy (font->encoding);
   gt1_free (font);
 }
 
@@ -3211,7 +3154,10 @@ gt1_get_glyph_outline (Gt1LoadedFont *font, int glyphnum, double *p_wx)
   Gt1Dict *charstrings;
   Gt1String *glyphcode;
 
-  glyphname = font->encoding[glyphnum & 0xff]; /* cheat! */
+  /* fixme: */
+  glyphname = GPOINTER_TO_INT (g_hash_table_lookup (font->encoding, GINT_TO_POINTER (glyphnum)));
+  if (!glyphname) return NULL;
+
   charstringsval = gt1_dict_lookup (font->fontdict, font->id_charstrings);
   charstrings = charstringsval->val.dict_val;
   glyphcodeval = gt1_dict_lookup (charstrings, glyphname);
@@ -3268,8 +3214,8 @@ gt1_get_kern_pair (Gt1LoadedFont *font, int glyph1, int glyph2)
 
   if (font == NULL)
     return 0;
-  name1 = font->encoding[glyph1 & 0xff];
-  name2 = font->encoding[glyph2 & 0xff];
+  name1 = GPOINTER_TO_INT (g_hash_table_lookup (font->encoding, GINT_TO_POINTER (glyph1)));
+  name2 = GPOINTER_TO_INT (g_hash_table_lookup (font->encoding, GINT_TO_POINTER (glyph2)));
   table_size = font->mfi->kern_pair_table_size;
   table = font->mfi->kern_pair_table;
   for (i = kern_pair_hash (name1, name2); idx = i % table_size,
